import numpy as np
from src.plot import plot_xy, plot_color2d
import time
from src.TiseSolverFgh2D import TiseSolverFgh2D
import os
import sys
from pathlib import Path

# BLAS ライブラリ用のスレッド数設定（使用しているライブラリに合わせて設定してください）
os.environ["MKL_NUM_THREADS"] = "4"  # Intel MKL を使用する場合
os.environ["OMP_NUM_THREADS"] = "4"  # OpenMP を使用する場合
os.environ["OPENBLAS_NUM_THREADS"] = "4"  # OpenBLAS を使用する場合

if __name__ == "__main__":
    # - Input parameters
    # -- Position
    xa = -6.0  # Position x = [xa, xb]
    xb = 6.0
    nx = 2**5  # Number of x points
    # -- Position
    ya = -6.0  # Position x = [xa, xb]
    yb = 6.0
    ny = 2**5  # Number of y points

    # -- How many eigenvalues and eigenvectors you want from the smallest eigenvalue? (num_eig < nx*ny)
    num_eig = 21

    # -- Output folder
    output_folder_path = "./out/"

    # -- Preparetion
    folder_path = Path(output_folder_path)  # Make folder
    folder_path.mkdir(exist_ok=True, parents=True)

    # =====================
    #         TISE
    # =====================

    # -- Solve TISE using Fourier-Grid Hamiltonian method
    solver = TiseSolverFgh2D(xa, xb, nx, ya, yb, ny, num_eig)
    x, y, Vxy, x_mesh, y_mesh = solver.abscissa_potential()
    start_time = time.time()  # Record the start time
    eigenvalues, eigenfunctions = solver.solve()
    end_time = time.time()  # Record the end time
    print(f"  Elapsed time TISE: {end_time - start_time:.6f} seconds")
    for i in range(15):
        print(i, eigenvalues[i])

    # -- Output TISE results
    n = 11
    z_mesh = eigenfunctions[:, :, n].real
    output_image_name = os.path.join(output_folder_path, f"eigenfunctions_n{n}.png")
    plot_color2d(
        x_mesh,
        y_mesh,
        z_mesh,
        color_range=(-0.5, 0.5),
        color_map_name="jet",
        show_gui=True,
        output_image_name=output_image_name,
    )

    # -- Upscaling using basis function
    x_new = np.linspace(xa, xb, 2**7, endpoint=False, dtype=np.float64)
    y_new = np.linspace(ya, yb, 2**7, endpoint=False, dtype=np.float64)
    x_new_mesh, y_new_mesh = np.meshgrid(x_new, y_new, indexing="ij")  # X, Y has shapes (len(x), len(y))
    z_new_mesh = solver.wf_upscale(n, x_new, y_new).real
    output_image_name = os.path.join(output_folder_path, f"eigenfunctions_n{n}_upsc.png")
    plot_color2d(
        x_new_mesh,
        y_new_mesh,
        z_new_mesh,
        color_range=(-0.5, 0.5),
        color_map_name="jet",
        aspect_ratio=1.0,
        show_gui=True,
        output_image_name=output_image_name,
    )

    sys.exit()
